<?php

namespace FiloBlu\Referrals\Observer;

use Magento\Framework\Event\ObserverInterface;

class SalesOrderAfterSave implements ObserverInterface
{
    const ORDER_STATUSES_TO_BE_MONITORED = 'filoblu_referrals/general/referral_order_statuses_to_monitor';

    protected $_referralHelper;
    protected $_emailHelper;

    public function __construct(
        \FiloBlu\Referrals\Helper\Core $referralHelper,
        \FiloBlu\Referrals\Helper\Email $emailHelper
    ) {
        $this->_referralHelper = $referralHelper;
        $this->_emailHelper = $emailHelper;
    }

    /**
     * After save observer for order
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return $this
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {

        // The module referral is not enabled, no need to go forward!
        if (!$this->_referralHelper->isEnabled()){
            //$this->_referralHelper->writeLog("FiloBlu\Referrals Module not enabled!");
            return $this;
        }

        $order = $observer->getEvent()->getOrder();
        $status = $order->getStatus();

        $allowed_order_statuses_to_be_monitored = $this->_referralHelper->getSysConfigValue(self::ORDER_STATUSES_TO_BE_MONITORED);
        $allowed_order_statuses_to_be_monitored = explode(',',$allowed_order_statuses_to_be_monitored);

        // Monitoring only the statuses I want
        if(in_array($status,$allowed_order_statuses_to_be_monitored)){

            $invited_by = $this->_referralHelper->whoInvitedCustomerId($order->getCustomerId());

            $this->_referralHelper->writeLog("COUPON ORDER COMPLETE: increment_id {$order->getIncrementId()} | customer_id {$order->getCustomerId()} | invited_by {$invited_by}");

            // If the user have not been invited by anyone stop here
            if(!$invited_by){
                return $this;
            }

            $invitations_of_this_user = $this->_referralHelper->getInvitationByDbField('referral_id',$order->getCustomerId(),false);

            $this->_referralHelper->writeLog("COUPON ORDER COMPLETE: increment_id {$order->getIncrementId()} | Invitations found {$invitations_of_this_user->getSize()}");

            // If the user have no invitations stop here
            if ($invitations_of_this_user->getSize() == 0){
                return $this;
            }

            // Reading all his invitations (should be always only one!)
            // One user can be invited only one time
            foreach ($invitations_of_this_user as $invitation) {
                // Check if already sent
                if($invitation->getWhoSentTheInviteCpSent() == 0 && $invitation->getStatus() == 'accepted'){
                    // Sending the coupon to who invited this customer
                    $result = $this->_emailHelper->notify_invitation_sender($invitation);
                    $this->_referralHelper->writeLog("COUPON ORDER COMPLETE: increment_id {$order->getIncrementId()} | Sending coupon to the sender of invitation_id {$invitation->getInvitationId()}");
                    if($result){
                        $this->_referralHelper->writeLog("COUPON ORDER COMPLETE: increment_id {$order->getIncrementId()} | Sended coupon!");
                        $invitation->setWhoSentTheInviteCpSent(1);
                        $invitation->save();
                    }
                    break;
                }
            }

        }

        return $this;
    }
}
