<?php

namespace FiloBlu\Referrals\Plugin;

class UtilityPlugin
{

    protected $_referralHelper;

    public function __construct(
        \FiloBlu\Referrals\Helper\Core $referralHelper
    )
    {
        $this->_referralHelper = $referralHelper;
    }

    /*
     * This is needed because ONLY the user receiving a coupon from an invite can use THAT coupon
     * That coupon is not shareable!
     */
    public function afterCanProcessRule($utility, $result, $rule, $address){

        // The validation is already not true, no need to go forward!
        if(!$result){
            return $result;
        }

        // The module referral is not enabled, no need to go forward!
        if (!$this->_referralHelper->isEnabled()){
            //$this->_referralHelper->writeLog("FiloBlu\Referrals Module not enabled!");
            return $result;
        }

        $couponCode = $address->getQuote()->getCouponCode();

        // The coupon is not a "invitational" coupon, no need to go forward!
        if (!$this->_referralHelper->isInvitationalTypeCoupon($couponCode)){
            return $result;
        }

        $customerId = $address->getQuote()->getCustomerId();

        // Remember, all coupons are unique!
        $invitation = $this->_referralHelper->getInvitationByCouponCode($couponCode);

        // There is no invitation with this coupon code! No need to go forward!
        if(!$invitation){
            $this->_referralHelper->writeLog("COUPON PLUGIN: Can't find invitation with coupon code {$couponCode}");
            return $result;
        }

        $this->_referralHelper->writeLog("COUPON PLUGIN: The customer_id {$customerId} is using the coupon {$couponCode}");

        // This customer $customerId, using this coupon $couponCode, is the sender of the invitation $invitation?
        $customerIsSender = false;
        if ($invitation->getCustomerId() == $customerId){
            $customerIsSender = true;
        }
        $customerIsReceiver = !$customerIsSender;


        //Now we have all we need to validate this coupon with the user
        $db_field = null;
        if($customerIsSender){
            $db_field = $this->_referralHelper->getCouponDbFieldNameForWhoSentTheInvite();
            $this->_referralHelper->writeLog("COUPON PLUGIN: The customer_id {$customerId} is the sender of the invitation_id {$invitation->getInvitationId()}");
        }
        if($customerIsReceiver){
            $db_field = $this->_referralHelper->getCouponDbFieldNameForWhoGotInvited();
            $this->_referralHelper->writeLog("COUPON PLUGIN: The customer_id {$customerId} is the receiver of the invitation_id {$invitation->getInvitationId()}");
        }

        $coupon_to_validate = $invitation->getData($db_field);

        // The coupon i'm trying to validate is not mine!!
        if ($couponCode != $coupon_to_validate){
            $this->_referralHelper->writeLog("COUPON PLUGIN: The coupon {$couponCode} does not belong to the customer_id {$customerId}");
            $rule->setIsValidForAddress($address, false);
            return false;
        }

        // All good!
        return $result;
    }

}